﻿#region namespaces

using System;
using System.Collections;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.TextManager.Interop;
using IOleServiceProvider =
    Microsoft.VisualStudio.OLE.Interop.IServiceProvider;

#endregion

[assembly: ComVisible(true)]

namespace at.jku.ssw.Coco.VSPlugin
{

  #region Attributes for Registry

  /// <summary>
  /// This is the class that implements the package exposed by this assembly.
  ///
  /// The minimum requirement for a class to be considered a valid package for Visual Studio
  /// is to implement the IVsPackage interface and register itself with the shell.
  /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
  /// to do it: it derives from the Package class that provides the implementation of the 
  /// IVsPackage interface and uses the registration attributes defined in the framework to 
  /// register itself and its components with the shell.
  /// </summary>
  [ProvideProjectItem(GuidList.guidCSPackageString, "Coco/R Grammar",
      "C:\\CocoProject\\Templates\\ProjectItems", 500)]
  [ProvideService(typeof (CocoLanguageService),
      ServiceName = "Coco/R Language Service")]
  [ProvideLanguageService(typeof (CocoLanguageService), "Coco/R Grammar",
      106, // resource ID of localized language name
      RequestStockColors = true, EnableLineNumbers = true, CodeSense = true
      , // Supports IntelliSense
      EnableCommenting = true // Supports commenting out code
      )]
  [ProvideLanguageExtension(typeof (CocoLanguageService), ".atg")]
  // This attribute tells the registration utility (regpkg.exe) that this class needs
  // to be registered as package.
  [PackageRegistration(UseManagedResourcesOnly = true)]
  // A Visual Studio component can be registered under different regitry roots; for instance
  // when you debug your package you want to register it in the experimental hive. This
  // attribute specifies the registry root to use if no one is provided to regpkg.exe with
  // the /root switch.
  [DefaultRegistryRoot("Software\\Microsoft\\VisualStudio\\9.0")]
  // This attribute is used to register the informations needed to show the this package
  // in the Help/About dialog of Visual Studio.
  [InstalledProductRegistration(false, "#110", "#112", "1.0",
      IconResourceID = 400)]
  // In order be loaded inside Visual Studio in a machine that has not the VS SDK installed, 
  // package needs to have a valid load key (it can be requested at 
  // http://msdn.microsoft.com/vstudio/extend/). This attributes tells the shell that this 
  // package has a load key embedded in its resources.
  [ProvideLoadKey("Standard", "1", "Coco/R VS 2008 Plugin",
      "ebner@silenzium.at", 113)]
  // This attribute is needed to let the shell know that this package exposes some menus.
  [ProvideMenuResource(1000, 1)]
  [Guid(GuidList.guidCocoPackagePkgString)]
  [ProvideOptionPage(typeof (OptionsPageGeneral), "CocoOptions", "General",
      101, 106, true)]
  [ProvideProfile(typeof (OptionsPageGeneral), "CocoOptions", "General",
      101, 106, true, DescriptionResourceID = 101)]

  #endregion

  public sealed class CocoPackage : Package
  {
    private CocoLanguageService cocoLanguageService;

    private BuildEvents buildEvents;
    private ErrorListProvider errorListProvider;

    protected override void Initialize()
    {
      base.Initialize();

      // Add Coco/R language service
      IServiceContainer serviceContainer = this;
      cocoLanguageService = new CocoLanguageService(this);
      cocoLanguageService.SetSite(this);
      serviceContainer.AddService(typeof (CocoLanguageService),
          cocoLanguageService, true);

      // Add build event handlers
      DTE2 dte = (DTE2) GetService(typeof (DTE));
      buildEvents = dte.Events.BuildEvents;
      buildEvents.OnBuildBegin += OnBuildBegin;
      buildEvents.OnBuildDone += OnBuildDone;

      // Create error provider
      errorListProvider = new ErrorListProvider(this);
      errorListProvider.Show();
    }

    private void NavigateDocument(object sender, EventArgs e)
    {
      ErrorTask task = sender as ErrorTask;
      if (task == null)
        throw new ArgumentException("sender");

      // Get the doc data for the task's document
      if (String.IsNullOrEmpty(task.Document))
      {
        return;
      }

      IVsUIShellOpenDocument openDoc =
          GetService(typeof (IVsUIShellOpenDocument)) as
              IVsUIShellOpenDocument;

      if (openDoc == null)
      {
        return;
      }

      IVsWindowFrame frame;
      IOleServiceProvider sp;
      IVsUIHierarchy hier;
      uint itemid;
      Guid logicalView = VSConstants.LOGVIEWID_Code;

      if (
          openDoc.OpenDocumentViaProject(task.Document, ref logicalView,
              out sp, out hier, out itemid, out frame) != VSConstants.S_OK
                  || frame == null)
      {
        return;
      }

      object docData;
      frame.GetProperty((int) __VSFPROPID.VSFPROPID_DocData, out docData);

      // Get the VsTextBuffer
      VsTextBuffer buffer = docData as VsTextBuffer;
      if (buffer == null)
      {
        IVsTextBufferProvider bufferProvider =
            docData as IVsTextBufferProvider;
        if (bufferProvider != null)
        {
          IVsTextLines lines;
          bufferProvider.GetTextBuffer(out lines);
          buffer = lines as VsTextBuffer;
          Debug.Assert(buffer != null,
              "IVsTextLines does not implement IVsTextBuffer");

          if (buffer == null)
          {
            return;
          }
        }
      }

      // Finally, perform the navigation.
      IVsTextManager mgr =
          GetService(typeof (VsTextManagerClass)) as IVsTextManager;

      if (mgr == null)
      {
        return;
      }

      mgr.NavigateToLineAndColumn(buffer, ref logicalView, task.Line,
          task.Column, task.Line, task.Column);
    }

    private void OnBuildBegin(vsBuildScope Scope, vsBuildAction Action)
    {
      errorListProvider.Tasks.Clear();
    }

    private void OnBuildDone(vsBuildScope Scope, vsBuildAction Action)
    {
      DTE2 dte = (DTE2) GetService(typeof (DTE));
      ErrorItems errors = dte.ToolWindows.ErrorList.ErrorItems;
      Hashtable mapCache = new Hashtable();

      for (uint i = 1; i <= errors.Count; i++)
      {
        ErrorItem error = errors.Item(i);
        string fn = Path.GetFileName(error.FileName);
        string dir = Path.GetDirectoryName(error.FileName);
        if (fn.ToLower() == "parser.cs")
        {
          string atgmap = Path.Combine(dir, "parser.atgmap");
          Mapping map = null;
          if (mapCache.ContainsKey(atgmap))
          {
            map = (Mapping) mapCache[atgmap];
          }
          else
          {
            if (File.Exists(atgmap))
            {
              map = new Mapping();
              map.Read(atgmap);
              mapCache[atgmap] = map;
            }
          }

          if (map != null)
          {
            int line, column;
            if (map.Get(error.Line - 1, error.Column - 1, out line,
                out column))
            {
              ErrorTask task = new ErrorTask();
              task.ErrorCategory = TaskErrorCategory.Error;
              task.Priority = TaskPriority.Normal;
              task.Text = error.Description;
              task.Column = column;
              task.Line = line;
              task.Document = map.Grammar;
              task.Navigate += NavigateDocument;
              errorListProvider.Tasks.Add(task);
            }
          }
        }
      }

      errorListProvider.Show();
    }
  }
}